#include <ctype.h>
#include "bmsfel_p.h"

//################################################
// IuWF1

//////////////////////////////////////////////////
// RXgN^Q
BmsObject::BmsObject() {
  this->set( '0', '0' );
}

BmsObject::BmsObject( int x ) {
  this->set( x );
}

BmsObject::BmsObject( const char *str ) {
  this->set( str );
}

BmsObject::BmsObject( char msb, char lsb ) {
  this->set( msb, lsb );
}

// fXgN^
BmsObject::~BmsObject() {
}

//////////////////////////////////////////////////
// setn
void BmsObject::set( int x ) {
  if ( x < 0 || BMS_OBJ_MAX < x ) { x = 0; }
  this->set( BmsUtil::int_to_hex36( x / HEX36_MAX ),
             BmsUtil::int_to_hex36( x % HEX36_MAX ) );
}

void BmsObject::set( const char *str ) {
  this->set( *(str), *(str + 1) );
}

void BmsObject::set( char msb, char lsb ) {
  if ( isalnum( msb ) == false ) { msb = '0'; }
  this->msb = (char)toupper( msb );
  if ( isalnum( lsb ) == false ) { lsb = '0'; }
  this->lsb = (char)toupper( lsb );
}

//////////////////////////////////////////////////
// getn
int BmsObject::get_int() const {
  return BmsUtil::hex36_to_int( this->msb, this->lsb );
}

char BmsObject::get_msb() const {
  return this->msb;
}

char BmsObject::get_lsb() const {
  return this->lsb;
}

char *BmsObject::get_str( char *str ) const {
  this->get_char( str, str + 1 );
  return str;
}

char *BmsObject::get_str_null( char *str ) const {
  this->get_str( str );
  str[2] = '\0';
  return str;
}

void BmsObject::get_char( char *msb, char *lsb ) const {
  *(msb) = this->msb;
  *(lsb) = this->lsb;
}


//////////////////////////////////////////////////
// ZQ
void BmsObject::operator ++() {
  this->succ();
}

void BmsObject::next() {
  this->succ();
}

void BmsObject::succ() {
  this->add( 1 );
}

void BmsObject::add( int num ) {
  int temp;
  temp = this->get_int() + ( num % BMS_OBJ_MAX );
  if( temp < 0           ) { temp += BMS_OBJ_MAX; }
  if( temp > BMS_OBJ_MAX ) { temp -= BMS_OBJ_MAX; }
  this->set( temp );
}

//////////////////////////////////////////////////
// ZQ
void BmsObject::operator --() {
  this->prev();
}

void BmsObject::prev() {
  this->sub( 1 );
}

void BmsObject::sub( int num ) {
  this->add( -num );
}

//////////////////////////////////////////////////
// rQ
bool BmsObject::operator ==( int x ) const {
  return this->equal( x );
}
bool BmsObject::operator !=( int x ) const {
  return !( this->equal( x ) );
}

bool BmsObject::operator ==( char *str ) const {
  return this->equal( str );
}
bool BmsObject::operator !=( char *str ) const {
  return !( this->equal( str ) );
}

bool BmsObject::operator ==( BmsObject obj ) const {
  return this->equal( obj );
}
bool BmsObject::operator !=( BmsObject obj ) const {
  return !( this->equal( obj ) );
}

bool BmsObject::equal( int x ) const {
  if ( x < 0 || BMS_OBJ_MAX < x ) { return false; }
  return this->equal( BmsUtil::int_to_hex36( x / HEX36_MAX ),
                      BmsUtil::int_to_hex36( x % HEX36_MAX ) );
}

bool BmsObject::equal( char *str ) const {
  return this->equal( *(str), *(str + 1) );
}

bool BmsObject::equal( BmsObject obj ) const {
  return this->equal( obj.msb, obj.lsb );
}

// ǂ==n͑SĂł
bool BmsObject::equal( char msb, char lsb ) const {
  return this->msb == msb && this->lsb == lsb;
}


//////////////////////////////////////////////////
// <= < >= >n  comparenĂ

//////////////////////////////////////////////////
// <=n
bool BmsObject::operator <=( int x ) const {
  return this->compare( x ) <= 0;
}

bool BmsObject::operator <=( char *str ) const {
  return this->compare( str ) <= 0;
}

bool BmsObject::operator <=( BmsObject obj ) const {
  return this->compare( obj ) <= 0;
}

//////////////////////////////////////////////////
// <n
bool BmsObject::operator <( int x ) const {
  return this->compare( x ) < 0;
}

bool BmsObject::operator <( char *str ) const {
  return this->compare( str ) < 0;
}

bool BmsObject::operator <( BmsObject obj ) const {
  return this->compare( obj ) < 0;
}

//////////////////////////////////////////////////
// >= n
bool BmsObject::operator >=( int x ) const {
  return this->compare( x ) >= 0;
}

bool BmsObject::operator >=( char *str ) const {
  return this->compare( str ) >= 0;
}

bool BmsObject::operator >=( BmsObject obj ) const {
  return this->compare( obj ) >= 0;
}

//////////////////////////////////////////////////
// >n
bool BmsObject::operator >( int x ) const {
  return this->compare( x ) > 0;
}

bool BmsObject::operator >( char *str ) const {
  return this->compare( str ) > 0;
}

bool BmsObject::operator >( BmsObject obj ) const {
  return this->compare( obj ) > 0;
}


//////////////////////////////////////////////////
// rĂ̍Ԃ
int BmsObject::compare( int x ) const {
  return this->get_int() - x;
}

int BmsObject::compare( char *str ) const {
  return this->compare( *(str), *(str + 1) );
}

int BmsObject::compare( BmsObject obj ) const {
  return this->compare( obj.msb, obj.lsb );
}

int BmsObject::compare( char msb, char lsb ) const {
  return this->compare( BmsUtil::hex36_to_int( msb, lsb ) );
}
